# -*- coding: utf-8 -*-
"""
Initialise a new k8sdc installation.

usage:
  k8sdc [--debug] init [--help | -h]  -p <provider>

options:
  -p, --provider=<provider>
                provider type. One of the following:
                  bare
                  vagrant
                  do [NOT IMPLEMENTED]
                  aws [NOT IMPLEMENTED]
                  federation [NOT IMPLEMENTED]
  -h, --help    show this help.
  --debug       show debug output.

example:
  k8sdc init -p vagrant
"""

import os
import sys
import shutil
import logging
from docopt import docopt
from k8sdc.provider import providers
from k8sdc.utility import copytree

logger = logging.getLogger(__name__)


class InitCmd(object):
  """Initialize a new k8sdc installation"""

  files       = ['site.yaml', 'LICENSE', 'ansible.cfg']
  directories = ['roles', 'group_vars', 'host_vars', 'playbooks', 'keys', 'utilities', 'charts']

  def parse(self, argv):
    args = docopt(__doc__, argv=argv)
    logger.debug("k8sdc init - args:\n{}".format(args))

    provider = args['--provider']
    if provider not in providers:
      logger.error("Unknown provider: {}".format(provider))
      logger.error("Permitted providers are: \n\t{}".format(providers.keys()))
      print(__doc__)
      sys.exit(1)
    self.provider = provider

  def run(self):
    curdir = os.getcwd()

    logger.debug('provider: {}'.format(self.provider))
    logger.debug('curdir:   {}'.format(curdir))
    logger.debug('----------')
    logger.info("Copying files for provider: {}".format(self.provider))

    # Check curdir does not include a provider.yaml file
    if os.path.exists(os.path.join(curdir, 'provider.yaml')):
      logger.error('Current directory already contains a \'provider.yaml\' file.')
      sys.exit(1)

    # Copy standard files
    for file in self.files:
      src = os.path.join(sys.prefix + '/k8sdc', file)
      logger.debug("Copying file: {}".format(file))
      shutil.copy2(src, curdir)
      logger.debug('----------')

    # Copy standard directories
    for directory in self.directories:
      src = os.path.join(sys.prefix + '/k8sdc', directory)
      logger.debug("Copying directory: {}".format(directory))
      copytree(src, os.path.join(curdir, directory))

    # TODO: Ensure keys have 0400 permissions!

    # Copy provider specific directory
    provider_dir = os.path.join('providers', self.provider)
    src = os.path.join(sys.prefix + '/k8sdc', provider_dir)
    logger.debug("Copying directory: {}".format(provider_dir))
    copytree(src, curdir)



# -*- coding: utf-8 -*-

import logging
import pprintpp
import os
import shlex
import shutil
import sys
import yaml
from ansible.executor.playbook_executor import PlaybookExecutor
from ansible.parsing.dataloader import DataLoader
from ansible.vars import VariableManager
from ansible.inventory import Inventory
from collections import namedtuple
from subprocess import Popen, PIPE, STDOUT, CalledProcessError

logger = logging.getLogger(__name__)


# Overide of shutil.copytree to cope with dest directory already existing
def copytree(src, dest, symlinks=False, ignore=None):
  logger.debug("src:  {}".format(src))
  logger.debug("dest: {}".format(dest))
  logger.debug('Copying files ...')
  if os.path.exists(dest):
    for item in os.listdir(src):
      s = os.path.join(src, item)
      d = os.path.join(dest, item)
      if os.path.isdir(s):
        shutil.copytree(s, d, symlinks, ignore)
      else:
        shutil.copy2(s, d)
  else:
    shutil.copytree(src, dest, symlinks, ignore)
  logger.debug('----------')


def load_yaml_file(yaml_file):
  yaml_file = os.path.normpath(yaml_file)
  pp        = pprintpp.PrettyPrinter(indent=2)

  if not os.path.exists(yaml_file):
    logger.error("Unable to find yaml file: {}".format(yaml_file))
    sys.exit(1)

  logger.info("Loading yaml file: {}".format(yaml_file))
  with open(yaml_file, 'r') as yaml_file:
    contents = yaml.safe_load(yaml_file)

  logger.debug('Yaml file contents as a Python object:\n' + pp.pformat(contents))

  return contents


def execute(command):
  cmd     = shlex.split(command)
  process = Popen(cmd, stdout=PIPE, stderr=STDOUT)

  while True:
    nextline = process.stdout.readline()
    if nextline == '' and process.poll() is not None:
      break
    logger.info(nextline.rstrip())

  output     = process.communicate()[0]
  returncode = process.returncode

  if returncode == 0:
    return output
  else:
    raise CalledProcessError(returncode = returncode,
                             cmd        = command,
                             output     = output)


def call_ansible(yaml_file, become=False, tag=None):
  """Call Ansible with a playbook."""

  variable_manager = VariableManager()
  loader           = DataLoader()
  inventory        = Inventory(loader, variable_manager)
  variable_manager.set_inventory(inventory)

  Options = namedtuple('Options',
                       ['listtags',
                        'listtasks',
                        'listhosts',
                        'syntax',
                        'connection',
                        'module_path',
                        'forks',
                        'remote_user',
                        'private_key_file',
                        'ssh_common_args',
                        'ssh_extra_args',
                        'sftp_extra_args',
                        'scp_extra_args',
                        'become',
                        'become_method',
                        'become_user',
                        'verbosity',
                        'check',
                        'tags'])

  options = Options(listtags=False,
                    listtasks=False,
                    listhosts=False,
                    syntax=False,
                    connection='',
                    module_path='',
                    forks=100,
                    remote_user='',
                    private_key_file=None,
                    ssh_common_args=None,
                    ssh_extra_args=None,
                    sftp_extra_args=None,
                    scp_extra_args=None,
                    become=become,
                    become_method='sudo',
                    become_user='root',
                    verbosity=2,
                    check=False,
                    tags=tag)

  pbex = PlaybookExecutor(playbooks=[yaml_file],
                          inventory=inventory,
                          variable_manager=variable_manager,
                          loader=loader,
                          options=options,
                          passwords={})

  logger.debug("Calling Ansible with yaml file: {}".format(yaml_file))
  result = pbex.run()
  if result:
    logger.error("An error occured whilst executing the Ansible Playbook.")
